#ifndef _EULER_HPP_
#define _EULER_HPP_

#include <random>
#include <array>
#include <cmath>
#include <iostream>
#include "gauss.hpp"



class Euler
{
private:
  double X0;
  double Xt;
  double Yt;
  double t;
  double dt;
public:
  
  double access_dt(void) const {return dt;}

  Euler(double dt, double X0=0) : X0(X0), Xt(X0), Yt(X0), t(0), dt(dt) {}

  void un_pas(void);

  std::array<double,2> XT_YT(double T);

  void reinitialisation(void)
  {
	Xt = X0;
	Yt = X0;
	t = 0;
  }

  void change_pas(double dt_nouveau)
  {
	reinitialisation();
	dt=dt_nouveau;
  }

  double erreur_faible(double T, int M);

  double erreur_forte(double T, int M);
};

std::array<double,2> Euler::XT_YT(double T)
{
  reinitialisation();
  while (t<T)
	un_pas();
  std::array<double,2> y;
  y[0] = Xt;
  y[1] = Yt;
  return y;
}



void Euler::un_pas(void)
{
  t += dt;
  Yt *= (1-dt);
  Xt *= exp(-dt);
  double G = gauss();
  Xt += sqrt(1-exp(-2*dt)) * G / sqrt(2);
  Yt += sqrt(2*(1-exp(-dt))/(1+exp(-dt))) * G + sqrt(dt-2*(1-exp(-dt))/(1+exp(-dt)))*gauss();
}




double Euler::erreur_forte(double T, int M)
{
  double moyenne = 0;
  std::array<double,2> tab;
  for (int i=0; i<M; i++)
	{
	  reinitialisation();
	  tab = XT_YT(T);
	  moyenne += std::abs(tab[1]-tab[0]) / M;
	}
  return moyenne;
}


#endif /* euler.hpp */
